/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.mock.junit.ComponentClientMockItf;

/**
 * Default mock of {@link AdminServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AdminServiceClientMock implements AdminServiceClient {

    private final Map<String, ComponentClientMockItf> bindingComponentClients;

    private final Map<String, ComponentClientMockItf> serviceEngineClients;

    public AdminServiceClientMock(final Map<String, ComponentClientMockItf> bindingComponentClients,
            final Map<String, ComponentClientMockItf> serviceEngineClients) {
        this.bindingComponentClients = bindingComponentClients;
        this.serviceEngineClients = serviceEngineClients;
    }

    // TODO: Remove this contructor when org.ow2.petals.jmx.api.mock.JMXClientMock will be no more used
    public AdminServiceClientMock() {
        this.bindingComponentClients = new HashMap<String, ComponentClientMockItf>();
        this.serviceEngineClients = new HashMap<String, ComponentClientMockItf>();
    }

    @Override
    public String getSystemInfo() throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ObjectName[] getBindingComponents() throws AdminServiceErrorException {
        return AdminServiceClientMock.getComponents(this.bindingComponentClients, "binding");
    }

    @Override
    public ObjectName[] getEngineComponents() throws AdminServiceErrorException {
        return AdminServiceClientMock.getComponents(this.serviceEngineClients, "engine");
    }

    private static final ObjectName[] getComponents(final Map<String, ComponentClientMockItf> componentClients,
            final String objectNameType) throws AdminServiceErrorException {

        final List<ObjectName> result = new ArrayList<ObjectName>(componentClients.size());
        for (final String componentName : componentClients.keySet()) {
            ObjectName objectName;
            try {
                objectName = new ObjectName("org.ow2.petals:type=" + objectNameType + ",name=" + componentName);
                result.add(objectName);
            } catch (final MalformedObjectNameException e) {
                // This exception should not occurs
                throw new AdminServiceErrorException(e);
            }
        }
        return result.toArray(new ObjectName[result.size()]);
    }

    @Override
    public ObjectName getComponentByName(String component) throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean isBinding(String componentName) throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public boolean isEngine(String componentName) throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public boolean isExchangeWithConsumerOkayForComponent(String containerName,
            ServiceEndpoint serviceEndpoint, MessageExchange exchange)
            throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public boolean isExchangeWithProviderOkayForComponent(String containerName,
            ServiceEndpoint serviceEndpoint, MessageExchange exchange)
            throws AdminServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

}
