/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ServiceAssemblyDoesNotExistException;
import org.ow2.petals.jmx.api.mock.junit.ServiceAssembly;

/**
 * Default mock of {@link DeploymentServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeploymentServiceClientMock implements DeploymentServiceClient {

    private final Map<URL, DeploymentOrder> serviceAssemblyDeploymentReports;

    /**
     * Service assemblies declared as deployed.
     */
    private final List<ServiceAssembly> serviceAssemblies;

    public DeploymentServiceClientMock(final Map<URL, DeploymentOrder> serviceAssemblyDeploymentReports,
            final List<ServiceAssembly> serviceAssemblies) {
        this.serviceAssemblyDeploymentReports = serviceAssemblyDeploymentReports;
        this.serviceAssemblies = serviceAssemblies;
    }

    @Override
    public String deploy(final URL packageURL) throws DeploymentServiceErrorException {
        final DeploymentOrder order = this.serviceAssemblyDeploymentReports.get(packageURL);
        if (order == null) {
            return null;
        } else {
            if (order.getDeploymentReport() != null) {
                return order.getDeploymentReport();
            } else {
                throw order.getError();
            }
        }
    }

    @Override
    public boolean forceUndeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        return true;
    }

    @Override
    public String[] getDeployedServiceAssemblies() throws DeploymentServiceErrorException {

        final List<String> saIdentifiers = new ArrayList();
        for (final ServiceAssembly serviceAssembly : this.serviceAssemblies) {
            saIdentifiers.add(serviceAssembly.getName());
        }
        return saIdentifiers.toArray(new String[0]);
    }

    @Override
    public String[] getDeployedServiceAssembliesForComponent(final String componentName)
            throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String getServiceAssemblyDescriptor(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        for (final ServiceAssembly serviceAssembly : this.serviceAssemblies) {
            if (serviceAssembly.getName().equals(serviceAssemblyName)) {
                try {
                    return serviceAssembly.getDescriptor();
                } catch (final JBIDescriptorException e) {
                    throw new DeploymentServiceErrorException(e);
                }
            }
        }
        // Here the requested service assembly was not found
        throw new DeploymentServiceErrorException(String.format("Service assembly not found: %s", serviceAssemblyName));
    }

    @Override
    public String[] getServiceUnitForServiceAssembly(final String serviceAssemblyName)
            throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public State getState(final String serviceAssemblyName) throws ServiceAssemblyDoesNotExistException,
            DeploymentServiceErrorException {
        for (final ServiceAssembly serviceAssembly : this.serviceAssemblies) {
            if (serviceAssembly.getName().equals(serviceAssemblyName)) {
                return serviceAssembly.getState();
            }
        }
        // Here the requested service assembly was not found
        throw new DeploymentServiceErrorException(String.format("Service assembly not found: %s", serviceAssemblyName));
    }

    @Override
    public boolean isServiceAssemblyDeployed(final String serviceAssembly) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String shutdown(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String[] shutdownAllServiceAssemblies() throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String start(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String[] startAllServiceAssemblies() throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String stop(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String[] stopAllServiceAssemblies() throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String undeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String[] undeployAllServiceAssemblies(final boolean force) throws DeploymentServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }
}
