/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.EndpointDirectoryServiceConstants;
import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.exception.EndpointDirectoryException;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;

/**
 * Mock of {@link TopologyServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class EndpointDirectoryClientMock implements EndpointDirectoryClient {

    public final static String EDP_CONT0_SVC0 = "HelloEndpoint-TargetContainer0";

    public final static String EDP_CONT1_SVC1 = "HelloEndpoint-TargetContainer1";

    public final static String SVC_NAMESPACE = "http://petals.ow2.org/service/";

    public final static String SVC0_LOCALPART = "HelloService0";

    public final static QName SVC0 = new QName(SVC_NAMESPACE, SVC0_LOCALPART);

    public final static String SVC1_LOCALPART = "HelloService1";

    public final static QName SVC1 = new QName(SVC_NAMESPACE, SVC1_LOCALPART);

    public final static String ITF_NAMESPACE = "http://petals.ow2.org/";

    public final static String ITF0_LOCALPART = "HelloPortType";

    public final static QName ITF0 = new QName(ITF_NAMESPACE, ITF0_LOCALPART);

    public final static String CONTAINER0 = "target-container-0";

    public final static String CONTAINER1 = "target-container-1";

    public final static String COMPONENT_NAME = "my-jbi-component";

    private final static List<Map<String, Object>> ALL_ENDPOINTS;

    static {
        ALL_ENDPOINTS = new ArrayList<>();

        final Map<String, Object> edp0Map = new HashMap<>();
        ALL_ENDPOINTS.add(edp0Map);
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.ENDPOINT_NAME, EDP_CONT0_SVC0);
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.TYPE,
                EndpointDirectoryServiceConstants.EndpointType.INTERNAL);
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.CONTAINER_NAME, CONTAINER0);
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.SERVICE_NAME, SVC0.toString());
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.INTERFACE_NAMES, new String[] { ITF0.toString() });
        edp0Map.put(EndpointDirectoryServiceConstants.Endpoint.COMPONENT_NAME, COMPONENT_NAME);

        final Map<String, Object> edp1Map = new HashMap<>();
        ALL_ENDPOINTS.add(edp1Map);
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.ENDPOINT_NAME, EDP_CONT1_SVC1);
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.TYPE,
                EndpointDirectoryServiceConstants.EndpointType.INTERNAL);
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.CONTAINER_NAME, CONTAINER1);
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.SERVICE_NAME, SVC1.toString());
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.INTERFACE_NAMES, new String[] { ITF0.toString() });
        edp1Map.put(EndpointDirectoryServiceConstants.Endpoint.COMPONENT_NAME, COMPONENT_NAME);
    }

    @Override
    public List<Map<String, Object>> getAllInternalEndpoints() throws EndpointDirectoryException {
        return null;
    }

    @Override
    public List<Map<String, Object>> getAllExternalEndpoints() throws EndpointDirectoryException {
        return Collections.unmodifiableList(ALL_ENDPOINTS);
    }

    @Override
    public List<Map<String, Object>> getAllEndpoints() throws EndpointDirectoryException {
        return Collections.unmodifiableList(ALL_ENDPOINTS);
    }

    @Override
    public String getDescription(String serviceName, String endpointName) throws EndpointDirectoryException {
        return null;
    }

}
