/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.mock.junit.ComponentClientMockItf;
import org.ow2.petals.jmx.api.mock.junit.SharedLibrary;

/**
 * Default mock of {@link InstallationServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InstallationServiceClientMock implements InstallationServiceClient {

    private final Map<SharedLibrary, List<ComponentClientMockItf>> sharedLibraries;

    private final Map<String, ComponentClientMockItf> bindingComponentClients;

    private final Map<String, ComponentClientMockItf> serviceEngineClients;

    public InstallationServiceClientMock(final Map<SharedLibrary, List<ComponentClientMockItf>> sharedLibraries,
            final Map<String, ComponentClientMockItf> bindingComponentClients,
            final Map<String, ComponentClientMockItf> serviceEngineClients) {
        this.sharedLibraries = sharedLibraries;
        this.bindingComponentClients = bindingComponentClients;
        this.serviceEngineClients = serviceEngineClients;
    }

    @Override
    public boolean forceUnloadInstaller(final String componentName) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName,
            final String sharedLibraryVersion)
            throws InstallationServiceErrorException {

        final List<String> result = new ArrayList<String>();
        final List<ComponentClientMockItf> components = this.sharedLibraries.get(new SharedLibrary(sharedLibraryName,
                sharedLibraryVersion));
        for (final ComponentClientMockItf component : components) {
            result.add(component.getIdentifier());
        }
        return result.toArray(new String[result.size()]);
    }

    @Override
    public String[][] getInstalledSharedLibraries() throws InstallationServiceErrorException {
        final List<String[]> result = new ArrayList<String[]>();
        for (final SharedLibrary sharedLibrary : this.sharedLibraries.keySet()) {
            final String[] entry = new String[2];
            entry[0] = sharedLibrary.getName();
            entry[1] = sharedLibrary.getVersion();
            result.add(entry);
        }
        return result.toArray(new String[result.size()][2]);
    }

    @Override
    public String[] getInstallers() throws InstallationServiceErrorException {
        final List<String> installers = new ArrayList<>(this.bindingComponentClients.keySet());
        installers.addAll(this.serviceEngineClients.keySet());
        return installers.toArray(new String[0]);
    }

    @Override
    public String installSharedLibrary(final URL sharedLibraryURL) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public InstallerComponentClient loadInstaller(final String componentName)
            throws InstallerComponentDoesNotExistException, InstallationServiceErrorException {
        boolean componentFound = false;
        for (final List<ComponentClientMockItf> componentsOrderedBySL : this.sharedLibraries.values()) {
            for (final ComponentClientMockItf component : componentsOrderedBySL) {
                if (component.getIdentifier().equals(componentName)) {
                    componentFound = true;
                }
            }
        }

        if (componentFound) {
            return new InstallerComponentClientMock();
        } else {
            throw new InstallerComponentDoesNotExistException(componentName);
        }
    }

    @Override
    public InstallerComponentClient loadNewInstaller(final URL packageURL) throws InstallationServiceErrorException {
        return new InstallerComponentClientMock();
    }

    @Override
    public String[] shutdownAllComponents() throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean shutdownComponent(final String componentName) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String[] startAllComponents() throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean startComponent(final String componentName) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String[] stopAllComponents() throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean stopComponent(final String componentName) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String[] uninstallAllComponents() throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean uninstallSharedLibrary(final String sharedLibraryName, final String sharedLibraryVersion)
            throws InstallationServiceErrorException {
        // TODO: Manage use case where a component using a shared library is always installed
        // TODO: Manage an inexisting SL
        this.sharedLibraries.remove(new SharedLibrary(sharedLibraryName, sharedLibraryVersion));
        return true;
    }

    @Override
    public String[] unloadAllInstallers(final boolean force) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean unloadInstaller(final String componentName) throws InstallationServiceErrorException {
        // TODO Auto-generated method stub
        return false;
    }

}
