/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import javax.management.MBeanServer;

import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;

/**
 * <p>Mock for unit tests of the Petals JMX Client factory.</p>
 * <p>The mock is controlled by connection parameters.</p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsJmxApiFactoryMock extends PetalsJmxApiFactory {

    /**
     * Host name to use when establishing connection to get the error
     * {@link ConnectionErrorException} on connection.
     */
    public final static String HOST_ERROR_CREATING_JMX_CLIENT = "error_creating_jmx_client";

    /**
     * Host name to use to have no error in processing.
     */
    public final static String HOST_NO_ERROR = "no_error";
    
    /**
     * A default port value to have no error in processing
     */
    public final static int DEFAULT_PORT = 7700;
    
    /**
     * A default username value to have no error in processing
     */
    public final static String DEFAULT_USERNAME = "no_error";
    
    /**
     * A default password value to have no error in processing
     */
    public final static String DEFAULT_PASSWORD = "no_error";

    @Override
    public JMXClient createJMXClient(final String host, final Integer jndiPort,
            final String username, final String password)
            throws ConnectionErrorException {
        if (HOST_ERROR_CREATING_JMX_CLIENT.equals(host)) {
            throw new ConnectionErrorException("Error creating the JMX client.");
        } else {
            return new JMXClientMock(host, jndiPort, username, password);
        }
    }

    @Override
    public JMXClient createJMXClient(final MBeanServer mbeanServer) throws ConnectionErrorException {
        throw new ConnectionErrorException("Error creating the JMX client, no supported operation.");
    }

}
