/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.junit;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.EndpointDirectoryServiceConstants;
import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.exception.EndpointDirectoryException;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;

public class EndpointDirectoryClientMock implements EndpointDirectoryClient {

    /**
     * Endpoints declared as deployed.
     */
    private final List<EndpointMock> endpoints;

    public EndpointDirectoryClientMock(final List<EndpointMock> endpoints) {
        this.endpoints = endpoints;
    }

    @Override
    public List<Map<String, Object>> getAllInternalEndpoints() throws EndpointDirectoryException {
        // TODO Auto-generated method stub
        return Collections.emptyList();
    }

    @Override
    public List<Map<String, Object>> getAllExternalEndpoints() throws EndpointDirectoryException {
        // TODO Auto-generated method stub
        return Collections.emptyList();
    }

    @Override
    public List<Map<String, Object>> getAllEndpoints() throws EndpointDirectoryException {
        final List<Map<String, Object>> results = new ArrayList<>();
        for (final EndpointMock endpoint : this.endpoints) {
            final Map<String, Object> endpointMap = new HashMap<>();
            endpointMap.put(EndpointDirectoryServiceConstants.Endpoint.ENDPOINT_NAME, endpoint.getEndpointName());
            endpointMap.put(EndpointDirectoryServiceConstants.Endpoint.SERVICE_NAME,
                    endpoint.getServiceName().toString());

            final QName[] interfaces = endpoint.getInterfaces();
            final List<String> convertedInterfaceNames = new ArrayList<>();
            if (interfaces.length > 0) {
                for (final QName interfaceName : interfaces) {
                    convertedInterfaceNames.add(interfaceName.toString());
                }
            } else {
                convertedInterfaceNames.add(EndpointDirectoryServiceConstants.Interface.UNDEFINED);
            }
            endpointMap.put(EndpointDirectoryServiceConstants.Endpoint.INTERFACE_NAMES,
                    convertedInterfaceNames.toArray(new String[convertedInterfaceNames.size()]));

            results.add(endpointMap);
        }
        return results;
    }

    @Override
    public String getDescription(final String serviceName, final String endpointName)
            throws EndpointDirectoryException {
        // TODO Auto-generated method stub
        return null;
    }

}
