/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.junit;

import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;

import org.junit.rules.ExternalResource;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.ComponentClient.State;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.configuration.component.InstallationConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.NoDefectListenerException;
import org.ow2.petals.jmx.api.mock.AdminServiceClientMock;
import org.ow2.petals.jmx.api.mock.DeploymentOrder;
import org.ow2.petals.jmx.api.mock.DeploymentServiceClientMock;
import org.ow2.petals.jmx.api.mock.InstallationServiceClientMock;

/**
 * The PetalsJmxApiJunitRule Rule allows creation of a mock implementation of Petals JMX API are guaranteed to be
 * deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class PetalsJmxApiJunitRule {
 *  &#064;Rule
 *  public PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule(host, port, username, password);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      jmxApi.setTopologyService(...);
 *      jmxApi.setPetalsAdminService(...);
 *      ...
 *  }
 * }
 * </pre>
 */
public class PetalsJmxApiJunitRule extends ExternalResource implements JMXClient {

    /**
     * <p>
     * Nature of a JBI component:
     * <ul>
     * <li>BINDING: a binding component,</li>
     * <li>ENGINE: a service engine.</li>
     * </ul>
     * </p>
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public enum ComponentType {
        BINDING, ENGINE;
    }

    /**
     * The {@link JMXConnectorServer} listen on the localhost
     */
    private final static String JMX_HOST = "localhost";

    /**
     * A user having a non-empty password
     */
    private final static String JMX_USER = "petals";

    private final static String JMX_PASSWORD = "petals";

    /**
     * The listening port of the {@link JMXConnectorServer}
     */
    private final int jmxPort;

    private final String jmxHost;

    private final String jmxUsername;

    private final String jmxPassword;

    private boolean isTopologyServiceClientExist = true;

    private boolean isInstallationServiceClientExist = true;

    private boolean isAdminServiceClientExist = true;

    private boolean isDeploymentServiceClientExist = true;

    /**
     * Binding components declared as installed.
     */
    private final Map<String, ComponentInstallerClientMockItf> installers = new HashMap<String, ComponentInstallerClientMockItf>();

    /**
     * Binding components declared as installed.
     */
    private final Map<String, ComponentClientMockItf> bindingComponentClients = new HashMap<String, ComponentClientMockItf>();

    /**
     * Service engines declared as installed.
     */
    private final Map<String, ComponentClientMockItf> serviceEngineClients = new HashMap<String, ComponentClientMockItf>();

    /**
     * Shared libraries declared as installed.
     */
    private final Map<SharedLibrary, List<ComponentClientMockItf>> sharedLibraries = new HashMap<SharedLibrary, List<ComponentClientMockItf>>();

    /**
     * Service assemblies declared as deployed.
     */
    private final List<ServiceAssembly> serviceAssemblies = new ArrayList<ServiceAssembly>();

    /**
     * Endpoints declared as deployed.
     */
    private final List<EndpointMock> endpoints = new ArrayList<>();

    private TopologyServiceClient topologyServiceClient;

    private final Map<URL, DeploymentOrder> serviceAssemblyDeploymentReports = new ConcurrentHashMap<URL, DeploymentOrder>();

    private final InstallationServiceClient installationServiceClient = new InstallationServiceClientMock(
            this.sharedLibraries, this.bindingComponentClients, this.serviceEngineClients);

    private final AdminServiceClient adminServiceClient = new AdminServiceClientMock(this.bindingComponentClients,
            this.serviceEngineClients);

    private final DeploymentServiceClient deploymentServiceClient = new DeploymentServiceClientMock(
            this.serviceAssemblyDeploymentReports, this.serviceAssemblies);

    private LocalTransporterMonitoringServiceClient localTransporterMonitoringServiceClient = null;

    private TcpTransporterMonitoringServiceClient tcpTransporterMonitoringServiceClient = null;

    private PetalsJmxApiFactory jmxApiFactory = null;

    private final OnConnectionHandler onConnectionHandler;

    /**
     * Creates a mock implementation of the Petals JMX API
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            Port on which the JMX server listen
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     * @param onConnectionHandler
     *            The connection handler simulating the connection establishment
     */
    public PetalsJmxApiJunitRule(final String hostname, final int jmxPort, final String username, final String password,
            final OnConnectionHandler onConnectionHandler) {
        this.jmxHost = hostname;
        this.jmxPort = jmxPort;
        this.jmxUsername = username;
        this.jmxPassword = password;
        this.onConnectionHandler = onConnectionHandler;
    }

    /**
     * Creates a mock implementation of the Petals JMX API
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            Port on which the JMX server listen
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     * @param isConnectionSucceeded
     *            If <code>true</code>, the connection is simulated as succeeded.
     */
    public PetalsJmxApiJunitRule(final String hostname, final int jmxPort, final String username, final String password,
            final boolean isConnectionSucceeded) {
        this(hostname, jmxPort, username, password, new OnConnectionHandler() {

            @Override
            public boolean onConnection() throws ConnectionErrorException {
                if (isConnectionSucceeded) {
                    return true;
                } else {
                    throw new ConnectionErrorException("Unable to connect to Petals container.");
                }
            }
        });
    }

    /**
     * Creates a mock implementation of the Petals JMX API, where the JMX connection will be simulated successfully
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            Port on which the JMX server listen
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     */
    public PetalsJmxApiJunitRule(final String hostname, final int jmxPort, final String username,
            final String password) {
        this(hostname, jmxPort, username, password, true);
    }

    /**
     * Creates a mock implementation of the Petals JMX API where the connection status is driven by
     * <code>isConnectionSucceeded</code> with default credentials:
     * <ul>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param isConnectionSucceeded
     *            If <code>true</code>, the connection is simulated as succeeded.
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public PetalsJmxApiJunitRule(final String hostname, final int jmxPort, final boolean isConnectionSucceeded) {
        this(hostname, jmxPort, JMX_USER, JMX_PASSWORD, isConnectionSucceeded);
    }

    /**
     * Creates a mock implementation of the Petals JMX API with default credentials:
     * <ul>
     * <li>connection status: succeeded,</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param hostname
     *            Hostname of the JMX server
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public PetalsJmxApiJunitRule(final String hostname, final int jmxPort) {
        this(hostname, jmxPort, true);
    }

    /**
     * Creates a mock implementation of the Petals JMX API where the connection status is driven by
     * <code>isConnectionSucceeded</code> with default values except for credentials:
     * <ul>
     * <li>host: localhost</li>
     * <li>port: an available port starting from {@value JMXClient#DEFAULT_PORT}</li>
     * </ul>
     * 
     * @param isConnectionSucceeded
     *            If <code>true</code>, the connection is simulated as succeeded.
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     */
    public PetalsJmxApiJunitRule(final String username, final String password, final boolean isConnectionSucceeded) {
        this(JMX_HOST, getNextAvailableJmxPort(JMXClient.DEFAULT_PORT), username, password, isConnectionSucceeded);
    }

    /**
     * Creates a mock implementation of the Petals JMX API with default values except for credentials:
     * <ul>
     * <li>connection status: succeeded,</li>
     * <li>host: localhost</li>
     * <li>port: an available port starting from {@value JMXClient#DEFAULT_PORT}</li>
     * </ul>
     * 
     * @param username
     *            Credential username required by the JMX server
     * @param password
     *            Credential password required by the JMX server
     */
    public PetalsJmxApiJunitRule(final String username, final String password) {
        this(username, password, true);
    }

    /**
     * Creates a mock implementation of the Petals JMX API where the connection status is driven by
     * <code>isConnectionSucceeded</code> with default values except for the JMX port:
     * <ul>
     * <li>host: localhost</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param isConnectionSucceeded
     *            If <code>true</code>, the connection is simulated as succeeded.
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public PetalsJmxApiJunitRule(final int jmxPort, final boolean isConnectionSucceeded) {
        this(JMX_HOST, jmxPort, isConnectionSucceeded);
    }

    /**
     * Creates a mock implementation of the Petals JMX API with default values except for the JMX port:
     * <ul>
     * <li>connection status: succeeded,</li>
     * <li>host: localhost</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param jmxPort
     *            The listening port of the {@link JMXConnectorServer}
     */
    public PetalsJmxApiJunitRule(final int jmxPort) {
        this(jmxPort, true);
    }

    /**
     * Creates a mock implementation of the Petals JMX API where the connection status is driven by
     * <code>isConnectionSucceeded</code> with default values:
     * <ul>
     * <li>host: localhost</li>
     * <li>port: an available port starting from {@value JMXClient#DEFAULT_PORT}</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     * 
     * @param isConnectionSucceeded
     *            If <code>true</code>, the connection is simulated as succeeded.
     */
    public PetalsJmxApiJunitRule(final boolean isConnectionSucceeded) {
        this(getNextAvailableJmxPort(JMXClient.DEFAULT_PORT), isConnectionSucceeded);
    }

    /**
     * Creates a mock implementation of the Petals JMX API with default values:
     * <ul>
     * <li>connection status: succeeded,</li>
     * <li>host: localhost</li>
     * <li>port: an available port starting from {@value JMXClient#DEFAULT_PORT}</li>
     * <li>username: petals</li>
     * <li>password:petals</li>
     * </ul>
     */
    public PetalsJmxApiJunitRule() {
        this(true);
    }

    private final static List<Integer> ALLOCATED_JMX_PORTS = new ArrayList<Integer>();

    private static int getNextAvailableJmxPort(final int startFrom) {
        synchronized (ALLOCATED_JMX_PORTS) {
            if (ALLOCATED_JMX_PORTS.contains(Integer.valueOf(startFrom))) {
                return getNextAvailableJmxPort(startFrom + 1);
            } else {
                ALLOCATED_JMX_PORTS.add(Integer.valueOf(startFrom));
                return startFrom;
            }
        }
    }

    @Override
    protected void before() throws Throwable {
        this.initializeJmxApi();
    }

    @Override
    protected void after() {
        try {
            this.disconnectAndDestroyJmxClient();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Initialize the mock implementation of the Petals JMX API
     */
    private void initializeJmxApi() throws Exception {

        System.setProperty(PetalsJmxApiFactory.PROPERTY_NAME,
                org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiFactory.class.getName());

        this.jmxApiFactory = PetalsJmxApiFactory.getInstance();
        if (jmxApiFactory instanceof org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiFactory) {
            ((org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiFactory) this.jmxApiFactory).registerJmxClient(this,
                    this.onConnectionHandler);
        } else {
            throw new Exception("The mock implementation of Petals JMX API is not correctly loaded: "
                    + jmxApiFactory.getClass().getName() + " instead of "
                    + org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiFactory.class.getName());
        }
    }

    /**
     * Free the mock implementation of the Petals JMX API
     */
    private void disconnectAndDestroyJmxClient() throws Exception {

        // Clear JBI environment
        this.bindingComponentClients.clear();
        this.serviceEngineClients.clear();
        this.sharedLibraries.clear();
        this.serviceAssemblies.clear();
        this.serviceAssemblyDeploymentReports.clear();

        this.disconnect();
        ((org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiFactory) this.jmxApiFactory).deregisterJmxClients();
        PetalsJmxApiFactory.close();
        System.clearProperty(PetalsJmxApiFactory.PROPERTY_NAME);
    }

    @Override
    public InetAddress getHost() throws NotRemoteJMXClientException {
        try {
            return InetAddress.getByName(this.jmxHost);
        } catch (final UnknownHostException e) {
            throw new Error(e);
        }
    }

    @Override
    public int getPort() throws NotRemoteJMXClientException {
        return this.jmxPort;
    }

    @Override
    public String getUsername() throws NotRemoteJMXClientException {
        return this.jmxUsername;
    }

    @Override
    public String getPassword() throws NotRemoteJMXClientException {
        return this.jmxPassword;
    }

    @Override
    public PetalsAdminServiceClient getPetalsAdminServiceClient() throws PetalsAdminDoesNotExistException,
            PetalsAdminServiceErrorException, ConnectionErrorException {
        return null;
    }

    @Override
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException, AdminServiceErrorException,
            ConnectionErrorException {
        if (!this.isAdminServiceClientExist) {
            throw new AdminDoesNotExistException();
        } else {
            return this.adminServiceClient;
        }
    }

    @Override
    public InstallationServiceClient getInstallationServiceClient() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, ConnectionErrorException {
        if (!this.isInstallationServiceClientExist) {
            throw new InstallationServiceDoesNotExistException("InstallationServiceClient does not exist");
        } else {
            return this.installationServiceClient;
        }
    }

    @Override
    public DeploymentServiceClient getDeploymentServiceClient() throws DeploymentServiceDoesNotExistException,
            DeploymentServiceErrorException, ConnectionErrorException {
        if (!this.isDeploymentServiceClientExist) {
            throw new DeploymentServiceDoesNotExistException();
        } else {
            return this.deploymentServiceClient;
        }
    }

    @Override
    public ComponentClient getComponentClient(final String componentName) throws ComponentDoesNotExistException,
            ComponentErrorException, ConnectionErrorException {
        return this.getComponentClient0(componentName);
    }

    private ComponentClientMockItf getComponentClient0(final String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        final ComponentClientMockItf bindingComponentClient = this.bindingComponentClients.get(componentName);
        if (bindingComponentClient == null) {
            final ComponentClientMockItf serviceEngineClient = this.serviceEngineClients.get(componentName);
            if (serviceEngineClient == null) {
                throw new ComponentDoesNotExistException(componentName);
            } else {
                return serviceEngineClient;
            }
        } else {
            return bindingComponentClient;
        }
    }

    @Override
    public ComponentClient getComponentClient(final ObjectName componentMBeanName)
            throws ComponentDoesNotExistException,
            ComponentErrorException, ConnectionErrorException {
        return this.getComponentClient(componentMBeanName.getKeyProperty("name"));
    }

    @Override
    public InstallerComponentClient getInstallerComponentClient(final String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        final ComponentInstallerClientMockItf mock = this.installers.get(componentName);
        if (mock == null) {
            throw new InstallerComponentDoesNotExistException(componentName);
        }
        return mock;
    }

    @Override
    public EndpointDirectoryClient getEndpointDirectoryClient() throws ConnectionErrorException,
            EndpointDirectoryServiceDoesNotExistException, EndpointDirectoryServiceErrorException {
        return new EndpointDirectoryClientMock(this.endpoints);
    }

    @Override
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient() throws ConnectionErrorException,
            SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        return null;
    }

    @Override
    public LoggerServiceClient getLoggerServiceClient() throws LoggerServiceErrorException,
            LoggerServiceDoesNotExistException, ConnectionErrorException {
        return null;
    }

    /**
     * {@inheritDoc}
     * 
     * If no Local transporter monitoring service client was previously registered, the error
     * {@link LocalTransporterMonitoringServiceDoesNotExistException} is thrown.
     */
    @Override
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        if (this.localTransporterMonitoringServiceClient == null) {
            throw new LocalTransporterMonitoringServiceDoesNotExistException();
        } else {
            return this.localTransporterMonitoringServiceClient;
        }
    }

    public void registerLocalTransporterMonitoringServiceClient(
            final LocalTransporterMonitoringServiceClient localTransporterMonitoringServiceClient) {
        this.localTransporterMonitoringServiceClient = localTransporterMonitoringServiceClient;
    }

    /**
     * {@inheritDoc}
     * 
     * If no TCP transporter monitoring service client was previously registered, the error
     * {@link TcpTransporterMonitoringServiceDoesNotExistException} is thrown.
     */
    @Override
    public TcpTransporterMonitoringServiceClient getTcpTransporterMonitoringServiceClient()
            throws TcpTransporterMonitoringServiceErrorException, TcpTransporterMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        if (this.tcpTransporterMonitoringServiceClient == null) {
            throw new TcpTransporterMonitoringServiceDoesNotExistException();
        } else {
            return this.tcpTransporterMonitoringServiceClient;
        }
    }

    public void registerTcpTransporterMonitoringServiceClient(
            final TcpTransporterMonitoringServiceClient tcpTransporterMonitoringServiceClient) {
        this.tcpTransporterMonitoringServiceClient = tcpTransporterMonitoringServiceClient;
    }

    @Override
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        try {
            final ComponentClientMockItf componentClientMock = this.getComponentClient0(componentIdentifier);
            if (componentClientMock != null) {
                final ComponentMonitoringServiceClient componentMonitoringServiceClient = componentClientMock
                        .getMonitoringServiceClient();
                if (componentMonitoringServiceClient == null) {
                    throw new ComponentMonitoringServiceDoesNotExistException();
                } else {
                    return componentMonitoringServiceClient;
                }
            } else {
                throw new ComponentMonitoringServiceDoesNotExistException();
            }
        } catch (final ComponentDoesNotExistException e) {
            throw new ComponentMonitoringServiceDoesNotExistException();
        } catch (final ComponentErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        }
    }

    @Override
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return null;
    }

    @Override
    public void subscribeToComponentMonitoringService(final String componentIdentifier,
            final DefectListener defectListener, final DefectFilter defectFilter) throws DefectListenerException,
            DefectEmitterDoesNotExistsException {
        // NOP
    }

    @Override
    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException, DefectListenerException {
        // NOP
    }

    @Override
    public TopologyServiceClient getTopologyServiceClient() throws TopologyServiceErrorException,
            TopologyServiceDoesNotExistException, ConnectionErrorException {
        if (!this.isTopologyServiceClientExist) {
            throw new TopologyServiceDoesNotExistException("TopologyServiceClient does not exist");
        } else {
            return this.topologyServiceClient;
        }
    }

    /**
     * <p>
     * If <code>isTopologyServiceClientExist</code> is set to <code>false</code>, the {@link TopologyServiceClient} will
     * be mocked as non-existent (ie. {@link JMXClient#getTopologyServiceClient()} will throw
     * {@link TopologyServiceDoesNotExistException}.
     * </p>
     * <p>
     * If <code>isTopologyServiceClientExist</code> is set to <code>true</code>, the mock {@link TopologyServiceClient}
     * set through {@link #setTopologyServiceClient(TopologyServiceClient)} is returned by
     * {@link JMXClient#getTopologyServiceClient()}.
     * </p>
     * 
     * @param isTopologyServiceClientExist
     *            Flag controlling the existence of a mock for {@link TopologyServiceClient}.
     */
    public void setTopologyServiceClientDoesNotExist(final boolean isTopologyServiceClientExist) {
        this.isTopologyServiceClientExist = isTopologyServiceClientExist;
    }

    /**
     * Set the mock of {@link TopologyServiceClient} returned by {@link JMXClient#getTopologyServiceClient()}
     * 
     * @param topologyServiceClient
     *            The mock of {@link TopologyServiceClient} to use
     */
    public void setTopologyServiceClient(final TopologyServiceClient topologyServiceClient) {
        this.topologyServiceClient = topologyServiceClient;
    }

    /**
     * <p>
     * If <code>isInstallationServiceClientExist</code> is set to <code>false</code>, the
     * {@link InstallationServiceClient} will be mocked as non-existent (ie.
     * {@link JMXClient#getInstallationServiceClient()} will throw {@link InstallationServiceDoesNotExistException}.
     * </p>
     * 
     * @param isInstallationServiceClientExist
     *            Flag controlling the existence of a mock for {@link InstallationServiceClient}.
     */
    public void setInstallationServiceClientDoesNotExist(final boolean isInstallationServiceClientExist) {
        this.isInstallationServiceClientExist = isInstallationServiceClientExist;
    }

    /**
     * Registers a shared library as if it was installed.
     * 
     * @param sharedLibrary
     *            The shared library to register
     */
    public void addSharedLibrary(final SharedLibrary sharedLibrary) throws InstallationServiceErrorException {
        if (!this.sharedLibraries.containsKey(sharedLibrary)) {
            this.sharedLibraries.put(sharedLibrary, new ArrayList<ComponentClientMockItf>());
        } else {
            throw new InstallationServiceErrorException("The shared library '" + sharedLibrary.getName() + "-"
                    + sharedLibrary.getVersion() + "' is already installed.");
        }
    }

    /**
     * <p>
     * Registers a component as if it was installed with the following characteristics:
     * <ul>
     * <li>a given identifier,</li>
     * <li>a given component type,</li>
     * <li>its state: STARTED.</li>
     * </ul>
     * </p>
     * 
     * @param componentName
     *            The component identifier associated to the JBI component to register
     * @param componentType
     *            The component type of the component to register
     * @return The mock associated to the {@link ComponentClient} of the component registered, so you can control its
     *         processing.
     */
    public ComponentClientMockItf addComponentClient(final String componentName, final ComponentType componentType)
            throws InstallationServiceErrorException {
        return this.addComponentClient(componentName, componentType, null);
    }

    /**
     * <p>
     * Registers a component as if it was installed with the following characteristics:
     * <ul>
     * <li>a given identifier,</li>
     * <li>a given component type,</li>
     * <li>a set of given shared libraries required,</li>
     * <li>its state: STARTED.</li>
     * </ul>
     * </p>
     * 
     * @param componentName
     *            The component identifier associated to the JBI component to register
     * @param componentType
     *            The component type of the component to register
     * @param sharedLibraries
     *            The shared libraries used by the component to register
     * @return The mock associated to the {@link ComponentClient} of the component registered, so you can control its
     *         processing.
     */
    public ComponentClientMockItf addComponentClient(final String componentName, final ComponentType componentType,
            final SharedLibrary[] sharedLibraries) throws InstallationServiceErrorException {

        return this.addComponentClient(componentName, componentType, sharedLibraries, null, null, null);
    }

    /**
     * <p>
     * Registers a component installer as if it was loaded with the following characteristics:
     * <ul>
     * <li>a given identifier,</li>
     * <li>a given component type,</li>
     * <li>a set of given shared libraries required,</li>
     * <li>a given installation configuration service client.</li>
     * </ul>
     * </p>
     * 
     * @param componentName
     *            The component identifier associated to the JBI component to register
     * @param componentType
     *            The component type of the component to register
     * @param runtimeConfigurationComponentClient
     *            The runtime configuration service associated to the component to register. Can be <code>null</code>,
     *            if no runtime configuration service will exist.
     * @return The mock associated to the {@link ComponentINstallerClient} of the component registered, so you can
     *         control its processing.
     */
    public ComponentInstallerClientMockItf addComponentInstallerClient(final String componentName,
            final ComponentType componentType,
            final InstallationConfigurationComponentClient installationConfigurationComponentClient,
            final Object installationConfigurationMock)
            throws InstallationServiceErrorException {

        final ComponentInstallerClientMockItf componentInstallerClientMock = new ComponentInstallerClientMock(
                componentName, componentType, installationConfigurationComponentClient, installationConfigurationMock);

        this.installers.put(componentName, componentInstallerClientMock);

        return componentInstallerClientMock;
    }

    /**
     * <p>
     * Registers a component as if it was installed with the following characteristics:
     * <ul>
     * <li>a given identifier,</li>
     * <li>a given component type,</li>
     * <li>a set of given shared libraries required,</li>
     * <li>a given runtime configuration service client,</li>
     * <li>its state: STARTED.</li>
     * </ul>
     * </p>
     * 
     * @param componentName
     *            The component identifier associated to the JBI component to register
     * @param componentType
     *            The component type of the component to register
     * @param sharedLibraries
     *            The shared libraries used by the component to register
     * @param runtimeConfigurationComponentClient
     *            The runtime configuration service associated to the component to register. Can be <code>null</code>,
     *            if no runtime configuration service will exist.
     * @param monitoringServiceClient
     *            The monitoring service associated to the component to register. Can be <code>null</code>, if no
     *            monitoring service will exist.
     * @return The mock associated to the {@link ComponentClient} of the component registered, so you can control its
     *         processing.
     */
    public ComponentClientMockItf addComponentClient(final String componentName, final ComponentType componentType,
            final SharedLibrary[] sharedLibraries,
            final RuntimeConfigurationComponentClient runtimeConfigurationComponentClient,
            final Object runtimeConfigurationMock,
            final ComponentMonitoringServiceClient monitoringServiceClient)
            throws InstallationServiceErrorException {

        final ComponentClientMockItf componentClientMock = new ComponentClientMock(componentName, componentType,
                State.STARTED, runtimeConfigurationComponentClient, runtimeConfigurationMock, monitoringServiceClient);

        if (componentType.equals(ComponentType.BINDING)) {
            this.bindingComponentClients.put(componentName, componentClientMock);
        } else {
            this.serviceEngineClients.put(componentName, componentClientMock);
        }

        if (sharedLibraries != null) {
            for (final SharedLibrary sharedLibrary : sharedLibraries) {
                final List<ComponentClientMockItf> components = this.sharedLibraries.get(sharedLibrary);
                if (components == null) {
                    throw new InstallationServiceErrorException("The required shared library '"
                            + sharedLibrary.getName() + "-" + sharedLibrary.getVersion() + "' is not installed.");
                } else {
                    components.add(componentClientMock);
                }
            }
        }

        return componentClientMock;
    }

    /**
     * <p>
     * Registers a service assembly as if it was deployed with the following characteristics:
     * <ul>
     * <li>a given identifier,</li>
     * <li>its state: STARTED.</li>
     * </ul>
     * </p>
     * 
     * @param serviceAssembly
     *            The service assembly artifact to register
     * @return The mock associated to the {@link ComponentClient} of the component registered, so you can control its
     *         processing.
     */
    public void addServiceAssembly(final ServiceAssembly serviceAssembly) throws DeploymentServiceErrorException {

        if (!this.serviceAssemblies.contains(serviceAssembly)) {
            this.serviceAssemblies.add(serviceAssembly);
        } else {
            throw new DeploymentServiceErrorException(
                    "The service assembly '" + serviceAssembly.getName() + "' is already deployed.");
        }
    }

    /**
     * <p>
     * If <code>isAdminServiceClientExist</code> is set to <code>false</code>, the {@link AdminServiceClient} will be
     * mocked as non-existent (ie. {@link JMXClient#getAdminServiceClient()} will throw
     * {@link AdminDoesNotExistException}.
     * </p>
     * 
     * @param isAdminServiceClientExist
     *            Flag controlling the existence of a mock for {@link AdminServiceClient}.
     */
    public void setAdminServiceClientDoesNotExist(final boolean isAdminServiceClientExist) {
        this.isAdminServiceClientExist = isAdminServiceClientExist;
    }

    /**
     * <p>
     * If <code>isDeploymentServiceClientExist</code> is set to <code>false</code>, the {@link DeploymentServiceClient}
     * will be mocked as non-existent (ie. {@link JMXClient#getDeploymentServiceClient()} will throw
     * {@link DeploymentServiceDoesNotExistException}.
     * </p>
     * 
     * @param isDeploymentServiceClientExist
     *            Flag controlling the existence of a mock for {@link DeploymentServiceClient}.
     */
    public void setDeploymentServiceClientDoesNotExist(final boolean isDeploymentServiceClientExist) {
        this.isDeploymentServiceClientExist = isDeploymentServiceClientExist;
    }

    /**
     * <p>
     * Established a connection between the JMX API (client part) and the JMX API mock (server part).
     * </p>
     * <p>
     * This method is an utility method that wraps
     * {@link PetalsJmxApiFactory#createJMXClient(String, Integer, String, String)}.
     * </p>
     * 
     * @return The JMX client connected
     */
    public JMXClient connect() throws ConnectionErrorException, DuplicatedServiceException, MissingServiceException {
        return PetalsJmxApiFactory.getInstance().createJMXClient(this.jmxHost, this.jmxPort, this.jmxUsername,
                this.jmxPassword);
    }

    @Override
    public void disconnect() throws ConnectionErrorException {
        // NOP
    }

    /**
     * Register a service-assembly deployment report to return when deploying the given service-assembly package.
     * 
     * @param saPackageUrl
     *            The service-assembly package
     * @param saDeploymentReport
     *            The service-assemby deployment report to return on deployment
     */
    public void registerServiceAssemblyDeployment(final URL saPackageUrl, final String saDeploymentReport) {
        this.serviceAssemblyDeploymentReports.put(saPackageUrl, new DeploymentOrder(saDeploymentReport));
    }

    /**
     * Register an error to return when deploying the given service-assembly package.
     * 
     * @param saPackageUrl
     *            The service-assembly package
     * @param error
     *            The error to return on deployment
     */
    public void registerServiceAssemblyDeployment(final URL saPackageUrl, final DeploymentServiceErrorException error) {
        this.serviceAssemblyDeploymentReports.put(saPackageUrl, new DeploymentOrder(error));
    }

    /**
     * Register an endpoint in the endpoint directory.
     * 
     * @param endpoint
     *            The endpoint to register
     */
    public void registerEndpoint(final EndpointMock endpoint) {
        this.endpoints.add(endpoint);
    }
}
