/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.monitoring;

import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

import org.ow2.petals.jmx.api.api.monitoring.defect.Defect;

/**
 * <p>
 * The default defect creator:
 * <ul>
 * <li>timestamp: the current timestamp,</li>
 * <li>sequence number: a generated sequence number incrementing a long counter,
 * </li>
 * <li>type: {@value #DEFECT_TYPE},</li>
 * <li>emitter name: {@value #DEFECT_EMITTER_NAME},</li>
 * <li>message: {@value #DEFECT_MESSAGE},</li>
 * <li>no user data.</li>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DefaultDefectCreator implements DefectCreator {

    public final static String DEFECT_TYPE = "defect.type";

    public final static String DEFECT_MESSAGE = "Defect message";

    public final static String DEFECT_EMITTER_NAME = "defect.emitter.name";

    private final AtomicLong seqNumber = new AtomicLong();

    @Override
    public Defect create() {
        return new Defect() {

            @Override
            public Object getUserData() {
                return null;
            }

            @Override
            public String getType() {
                return DEFECT_TYPE;
            }

            @Override
            public long getTimestamp() {
                return new Date().getTime();
            }

            @Override
            public long getSequenceNumber() {
                return DefaultDefectCreator.this.seqNumber.getAndIncrement();
            }

            @Override
            public String getMessage() {
                return DEFECT_MESSAGE;
            }

            @Override
            public String getEmitter() {
                return DEFECT_EMITTER_NAME;
            }
        };
    }

}
