/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.monitoring;

import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;

/**
 * A default defect emitter that emits a defect every 500 ms.
 * 
 * @author Christophe DENEUX
 * 
 */
public class DefaultDefectEmitterMock extends Thread {

    public final static long PERIOD = 500;

    /**
     * A defect creator
     */
    private final DefectCreator defectCreator;

    /**
     * The defect listener to which the defect must be sent.
     */
    private final DefectListener defectListener;

    public DefaultDefectEmitterMock(final DefectCreator defectCreator,
            final DefectListener defectListener) {
        this.defectCreator = defectCreator;
        this.defectListener = defectListener;
    }

    @Override
    public void run() {

        try {
            while (true) {
                Thread.sleep(PERIOD);
                this.defectListener.onDefect(defectCreator.create());
            }
        } catch (final InterruptedException e) {
            // NOP
        }
    }
}
