/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.monitoring.component.framework;

import java.util.Map;

import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.mock.junit.monitoring.object.ObjectPoolMetricsMock;
import org.ow2.petals.jmx.api.mock.junit.monitoring.object.ThreadPoolMetricsMock;

/**
 * Mock of {@link ComponentMonitoringServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ComponentMonitoringServiceClientMock implements ComponentMonitoringServiceClient {

    /**
     * Component name to use to mock an unexisting component.
     */
    public final static String IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT = "IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT";

    /**
     * Component name to use to mock a default component.
     */
    public final static String DEFAULT_COMPONENT_IDENTIFIER = "DEFAULT_COMPONENT_IDENTIFIER";

    //
    // Mex Acceptor Thread Pool
    //

    /**
     * Value returns by the method {@link #getMexAcceptorThreadPoolActiveThreadsMaxSize()}. Must be set by the unit test
     * if needed.
     */
    public static Integer MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;

    /**
     * Value returns by the method {@link #getMexAcceptorThreadPoolActiveThreadsCurrentSize()}. Must be set by the unit
     * test if needed.
     */
    public static Integer MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;

    /**
     * Value returns by the method {@link #getMexAcceptorWorkingMax()}. Must be set by the unit test if needed.
     */
    public static Long MEX_ACCEPTOR_WORKING_NB_MAX = null;

    /**
     * Value returns by the method {@link #getMexAcceptorWorkingCurrent()}. Must be set by the unit test if needed.
     */
    public static Long MEX_ACCEPTOR_WORKING_NB_CURRENT = null;

    /**
     * Value returns by the method {@link #getMexAcceptorRelativeDurations()}. Must be set by the unit test if needed.
     */
    public static Long[] MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS = null;

    //
    // Mex Processor Object Pool
    //

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getBorrowedObjectMaxNumber()}. Must be set by the unit
     * test if needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getBorrowedObjectNumber()}. Must be set by the unit test
     * if needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getIdleObjectMaxNumber()}. Must be set by the unit test
     * if needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getIdleObjectNumber()}. Must be set by the unit test if
     * needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getMaxSize()}. Must be set by the unit test if needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getMinIdleSize()}. Must be set by the unit test if
     * needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = null;

    /**
     * Value returns by the method {@link ObjectPoolMetricsMock#getExhaustions()}. Must be set by the unit test if
     * needed.
     */
    public static Long MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = null;

    //
    // Mex Processor Thread Pool
    //

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getActiveThreadsMaxSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getActiveThreadsSize()}. Must be set by the unit test if
     * needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getIdleThreadsMaxSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getIdleThreadsSize()}. Must be set by the unit test if
     * needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getMaxSize()}. Must be set by the unit test if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getMinSize()}. Must be set by the unit test if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getEnqueuedRequestsSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getEnqueuedRequestsMaxSize()}. Must be set by the unit
     * test if needed.
     */
    public static Long MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = null;

    /**
     * Value returns by the method {@link #getServiceProviderInvocationsCount()} . Must be set by the unit test if
     * needed.
     */
    public static Map<ServiceProviderInvocationKey, Long> SERVICE_PROVIDER_INVOCATIONS_COUNT = null;

    /**
     * Value returns by the method {@link #getServiceProviderInvocationsResponseTimes()}. Must be set by the unit test
     * if needed.
     */
    public static Map<ServiceProviderInvocationKey, Long[]> SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;

    /**
     * Component name to use to mock a {@link ComponentMonitoringServiceClient} generating error getting monitoring
     * values.
     */
    public final static String IDENTIFIER_FOR_COMPONENT_WITH_ERROR = "IDENTIFIER_FOR_COMPONENT_WITH_ERROR";

    /**
     * Value returns by the method {@link #getServiceProviderInvocationsResponseTimesGroupedByItfSvcOpAndExecStatus()} .
     * Must be set by the unit test if needed.
     */
    public static Map<ServiceProviderInvocationKey, Long[]> SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS;

    protected final static String ERROR_MESSAGE_GETTING_MONITORING_VALUES = "An error occurs getting monitoring value";

    protected final String componentName;

    public ComponentMonitoringServiceClientMock(final String componentName) {
        this.componentName = componentName;
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long> getServiceProviderInvocationsCount()
            throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return SERVICE_PROVIDER_INVOCATIONS_COUNT;
        }
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimes()
            throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES;
        }
    }

    @Override
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimesGroupedByItfSvcOpAndExecStatus()
            throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS;
        }
    }

    @Override
    public Integer getMexAcceptorThreadPoolActiveThreadsMaxSize() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX;
        }
    }

    @Override
    public Integer getMexAcceptorThreadPoolActiveThreadsCurrentSize() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT;
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the content of following values are returned:
     * <ul>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_MIN_SIZE},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_MAX_SIZE},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT},</li>
     * <li>{@link #MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX}.</li>
     * </ul>
     * </p>
     */
    @Override
    public ThreadPoolMetrics getMexProcessorThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {

        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {

            return new ThreadPoolMetricsMock(MEX_PROCESSOR_THREAD_POOL_MIN_SIZE, MEX_PROCESSOR_THREAD_POOL_MAX_SIZE,
                    MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT, MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX,
                    MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT, MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX,
                    MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT, MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the value {@link #MEX_ACCEPTOR_WORKING_NB_CURRENT}
     * is returned.
     * </p>
     */
    @Override
    public Long getMexAcceptorWorkingCurrent() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return MEX_ACCEPTOR_WORKING_NB_CURRENT;
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the value {@link #MEX_ACCEPTOR_WORKING_NB_MAX} is
     * returned.
     * </p>
     */
    @Override
    public Long getMexAcceptorWorkingMax() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return MEX_ACCEPTOR_WORKING_NB_MAX;
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the value
     * {@link #MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS} is returned.
     * </p>
     */
    @Override
    public Long[] getMexAcceptorRelativeDurations() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return MEX_ACCEPTOR_WORKING_RELATIVE_DURATIONS;
        }
    }
}
